package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention;

/*******************************************************************************
 * Copyright (c) 2007, 2010 Wind River Systems, Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Wind River Systems - initial API and implementation
 *******************************************************************************/

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response.*;
import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.IRM;
import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Strategy.strategiesList;
import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IPeer;
import org.eclipse.tm.tcf.protocol.Protocol;
import org.eclipse.tm.tcf.services.IPathMap.PathMapRule;

/**
 * TCF Test Suite implements stress testing of communication channels and capabilities of remote peer.
 * It is intended to be used before starting a debug session for a first time to make sure the selected
 * target is stable and reliable.
 */
public class Prepared {

    private final static int NUM_CHANNELS = 1;

    private final RMListener listener;
    private final IChannel[] channels;

    private final LinkedList<Runnable> pending_plans = new LinkedList<Runnable>();
    private final Collection<Throwable> errors = new ArrayList<Throwable>();
    //private final Map<IRM,IChannel> active_plans = new HashMap<IRM,IChannel>();
    // private final LinkedList<IRM> active_plans = new LinkedList<IRM>();
    private final LinkedList<IReactiveMeasures> active_plans = new LinkedList<IReactiveMeasures>();

    private int count_total;
    private int count_done;
    private String RM_Name;
    private String RM_parameter;
    
    boolean cancel;
    boolean canceled;
    boolean target_lock;

    public interface RMListener {
        public void progress(String label, int done, int total);
        public void done(Collection<Throwable> errors);
    }
    public Prepared(final IPeer peer, final RMListener listener, final List<PathMapRule> path_map, final  LinkedList<strategiesList> select_plans) throws IOException {
        this.listener = listener;
       
        	pending_plans.add(new Runnable() {
                public void run() {
                    int i;
                	int size = select_plans.size();

                    for (i=0; i< size;i++ ) {
                    	RM_Name = select_plans.getFirst().RMName;
                    	RM_parameter = select_plans.getFirst().RMParameter;

                        listener.progress("Adding Reactive Measures["+ RM_Name +"] from category[] to pending list ...", ++count_done, count_total);
                        for (IChannel channel : channels) {
                        	if (RM_Name == "RM_REBOOT") {
                    				active_plans.add(new RMReboot(channel));
                        	}else if (RM_Name == RMKillProcess.Name) {
                        			active_plans.add(new RMKillProcess(channel,RM_parameter));
                        	}else if (RM_Name == RMShutdown.Name) {
                    				active_plans.add(new RMShutdown(channel));
                        	}else if (RM_Name == RMReboot.Name) {
                    				active_plans.add(new RMReboot(channel));
                        	}else if (RM_Name == RMLogout.Name) {
                    				active_plans.add(new RMLogout(channel,RM_parameter));
                        	}else if (RM_Name == RMRemoveUser.Name) {
                    				active_plans.add(new RMRemoveUser(channel,RM_parameter));
                        	}else if (RM_Name == RMRemoveUserAndAllDir.Name) {
                    				active_plans.add(new RMRemoveUserAndAllDir(channel,RM_parameter));
                        	}else if (RM_Name == RMLockUser.Name) {
                    				active_plans.add(new RMLockUser(channel,RM_parameter));
                        	}else if (RM_Name == RMChangeFileOwnership.Name) {
                    				active_plans.add(new RMChangeFileOwnership(channel,RM_parameter));
                        	}else if (RM_Name == RMMaxDaysPassUse.Name) {
                    				active_plans.add(new RMMaxDaysPassUse(channel));
                        	}else if (RM_Name == RMMinDaysBetweenPassChange.Name) {
                    				active_plans.add(new RMMinDaysBetweenPassChange(channel));
                        	}else if (RM_Name == RMNumDaysWarningPassExpire.Name) {
                    				active_plans.add(new RMNumDaysWarningPassExpire(channel));
                        	}else if (RM_Name == RMPerDropPingPackets.Name) {
                    				active_plans.add(new RMPerDropPingPackets(channel));
                        	}else if (RM_Name == RMPerDeactiveIpForwarding.Name) {
                    				active_plans.add(new RMPerDeactiveIpForwarding(channel));
                        	}else if (RM_Name == RMTranDeactiveIpForwarding.Name) {
                    				active_plans.add(new RMPerDeactiveIpForwarding(channel));
                        	}else if (RM_Name == RMTranDropPingPackets.Name) {
                    				active_plans.add(new RMPerDeactiveIpForwarding(channel));
                        	}
                        }
                        select_plans.removeFirst();
                    }
                }
            });
 
//        pending_plans.add(new Runnable() {
//            public void run() {
//                listener.progress("Running Interability Test...", ++count_done, count_total);
//                for (int i = 0; i < channels.length; i++) {
//                    IRM test = null;
//                    switch (i % 4) {
//                    case 0: test = new TestEcho(Prepared.this, channels[i]); break;
//                    case 1: test = new TestAttachTerminate(Prepared.this, channels[i]); break;
//                    case 2: test = new TestRCBP1(Prepared.this, channels[i], i); break;
//                    case 3: test = new TestFileSystem(Prepared.this, channels[i], i); break;
//                    }
//                    active_plans.put(test, channels[i]);
//                }
//            }
//        });
        count_total = pending_plans.size() * 2;
        channels = new IChannel[NUM_CHANNELS];
        Protocol.invokeLater(new Runnable() {
            public void run() {
                try {
                    openChannels(peer);
                }
                catch (Throwable x) {
                    errors.add(x);
                    int cnt = 0;
                    for (int i = 0; i < channels.length; i++) {
                        if (channels[i] == null) continue;
                        if (channels[i].getState() != IChannel.STATE_CLOSED) channels[i].close();
                        cnt++;
                    }
                    if (cnt == 0) listener.done(errors);
                }
            }
        });
    }

    private void openChannels(IPeer peer) {
        listener.progress("Openning communication channels...", count_done, count_total);
        for (int i = 0; i < channels.length; i++) {
            final IChannel channel = channels[i] = peer.openChannel();
            channel.addChannelListener(new IChannel.IChannelListener() {

                public void onChannelOpened() {
                    for (int i = 0; i < channels.length; i++) {
                        if (channels[i] == null) return;
                        if (channels[i].getState() != IChannel.STATE_OPEN) return;
                    }
                    runNextPlan();
                }

                public void congestionLevel(int level) {
                }

                public void onChannelClosed(Throwable error) {
                    channel.removeChannelListener(this);
                    if (error == null && errors.isEmpty() && (!active_plans.isEmpty() || !pending_plans.isEmpty()) && !cancel) {
                        error = new IOException("Remote peer closed connection before all tests finished");
                    }
                    int cnt = 0;
                    for (int i = 0; i < channels.length; i++) {
                        if (channels[i] == channel) {
                            channels[i] = null;
                            if (error != null && errors.isEmpty()) errors.add(error);
                            //for (Iterator<IRM> n = active_plans.keySet().iterator(); n.hasNext();) {
                            //    if (active_plans.get(n.next()) == channel) n.remove();
                            //}
                        }
                        if (channels[i] == null) continue;
                        if ((error != null || active_plans.isEmpty() && pending_plans.isEmpty()) &&
                                channels[i].getState() != IChannel.STATE_CLOSED) channels[i].close();
                        cnt++;
                    }
                    if (cnt == 0) listener.done(errors);
                }
            });
        }
    }

    public void cancel() {
        cancel = true;
        if (canceled) return;
//        for (final IRM t : active_plans.keySet()) {
//            if (t instanceof TestRCBP1) {
//                ((TestRCBP1)t).cancel(new Runnable() {
//                    public void run() {
//                        assert active_plans.get(t) == null;
//                        cancel();
//                    }
//                });
//                return;
//            }
//        }
//        canceled = true;
//        for (IChannel c : channels) {
//            if (c != null && c.getState() != IChannel.STATE_CLOSED) c.close();
//        }
    }

    public boolean isCanceled() {
        return canceled;
    }

    public boolean isActive(IRM test) {
    	return true;
        //return active_plans.get(test) != null;
    }

    public void done(IRM test, Throwable error) {
       // assert active_plans.get(test) != null;
        if (error != null && !canceled) errors.add(error);
        active_plans.remove(test);
        if (active_plans.isEmpty()) runNextPlan();
    }

    private void runNextPlan() {
        while (!pending_plans.isEmpty()) {
            listener.progress(null, ++count_done, count_total);
            pending_plans.removeFirst().run();
            while (!active_plans.isEmpty()) {
            	//IRM test;
            	IReactiveMeasures test;
            	test = active_plans.getFirst();
            	test.start();
            	active_plans.removeFirst();
            }
            active_plans.clear();
        }
        
    }
}
