package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.IRM;
import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Prepared;
import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IToken;
import org.eclipse.tm.tcf.protocol.Protocol;
import org.eclipse.tm.tcf.services.IFileSystem;
import org.eclipse.tm.tcf.services.IProcesses;
import org.eclipse.tm.tcf.services.IProcesses.DoneStart;
import org.eclipse.tm.tcf.services.IProcesses.ProcessContext;


public class RMProcesses implements IRM{
	public static  int
    	PLAN_KILL_PROCESS = 1,
    	PLAN_REBOOT = 2,
		PLAN_SHUTDOWN = 3,
    	PLAN_LOGOUT = 4,
		PLAN_REMOVE_USER = 5,					 // remove user from the /etc/passwd and /etc/shadow  
		PLAN_LOCK_USER = 6,                      // change user's password
		PLAN_REMOVE_USER_AND_ALL_DIR = 7,        // remove user and all the contents of the user's home directory
		PLAN_CHANGE_FILE_OWNERSHIP = 8,          // change the ownership of a file to root
		PLAN_PASS_MAX_DAYS = 9,      			 // Maximum number of days a password may be used: (PASS_MAX_DAYS) (/etc/login.defs)
		PLAN_MIN_DAYS = 10,            // Minimum number of days allowed between password changes: (PASS_MIN_DAYS) (/etc/login.defs)
		PLAN_WARN_AGE = 11,            // Number of days warning given before a password expires: (PASS_WARN_AGE) (/etc/login.defs)
		PLAN_ALLOWED_HOSTS = 12,	   // Changing list of hosts that are allowed to access the system: (/etc/hosts.allow)
									   // It has a format to add a rule: (ALL : ALL : allow)
		PLAN_NOT_ALLOWED_HOSTS = 13;   // Changing list of hosts that are not allowed to access the system: (/etc/hosts.deny)
		 							   // It has a format to add a rule: (ALL: PARANOID)
				

	private final Prepared test_suite;
    private final int channel_id;
    private final int plan_id;

    private final IProcesses  processes;
    
    private boolean signal_sent;

	
	public RMProcesses(Prepared test_suite, IChannel channel, int channel_id, int plan_id) {
        this.test_suite = test_suite;
        this.channel_id = channel_id;
        this.plan_id = plan_id;
        
        processes = channel.getRemoteService(IProcesses.class);
         
    }
	
	@Override
	public void start() {
		// TODO Auto-generated method stub
		if (processes == null) {
            test_suite.done(this, null);
        }
        else {
        	switch (plan_id) {
		          case 1:
		        	  kill();
		        	  break;
		          case 2:
		        	  reboot();
		        	  break;
		          case 3:
		        	  shutdown();
		        	  break;
		          case 4:
		        	  logout();
		        	  break;
		          case 5:
		        	  remove_user();
		        	  break;
		          case 6:
		        	  lock_user();
		        	  break;
		          case 7:
		        	  remove_user_dir();
		        	  break;
		          case 8:
		        	  change_file_ownership();
		        	  break;
		          case 9:
		        	  pass_max_days();
		        	  break;
		          case 10:
		        	  min_days();
		        	  break;
		          case 11:
		        	  warn_age();
		        	  break;
		          case 12:
		        	  allowed_hosts();
		        	  break;
		          case 13:
		        	  not_allowed_hosts();
		        	  break;
  	  
        	}
        }
	}
	
	public void kill() {
		int code = 9;
    	processes.signal("P6262", code, new IProcesses.DoneCommand() {
            public void doneCommand(IToken token, Exception error) {
                if (error != null) {
                    exit(error);
                }
                else {
                    signal_sent = true;
                }
            }
        });
		System.out.println("Kill Plan was run");
	}
	
	public void reboot() {
		  final String dir = "/home/";
		  final String file = "/sbin/reboot";
	      final String args = "";
	      
      
          boolean attach = false;
          final HashMap<String,String> process_env = new HashMap<String,String>();
       
          processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
		  });
		  System.out.println("reboot Plan was run");
    }
	
	public void logout() {
		  final String dir = "/home/";
		  final String file = "/usr/bin/pkill";
	      final String args = "-KILL -u alireza";
	      
    
          processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		  System.out.println("logout Plan was run");
    }
	
	public void shutdown() {
		  final String dir = "/home/";
		  final String file = "/sbin/shutdown";
	      final String args = "";
	      
    
          boolean attach = false;
          final HashMap<String,String> process_env = new HashMap<String,String>();
     
          processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
		  });
		  System.out.println("shutdown Plan was run");
   }
	
	public void remove_user() {
		final String dir = "/home/";
		final String file = "/usr/sbin/userdel";
	    final String args = "sajad";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("remove_user Plan was run");
    }
	
	public void remove_user_dir() {
		final String dir = "/home/";
		final String file = "/usr/sbin/userdel";
	    final String args = "-r sajad";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("remove_user_dir Plan was run");
    }
	
	public void lock_user() {
		final String dir = "/home/";
		final String file = "/usr/bin/passwd";
	    final String args = "-l sajad";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("lock_user Plan was run");
    }
	
	public void change_file_ownership() {
		final String dir = "/home/";
		final String file = "/bin/chown";
	    final String args = "root:users /home/alireza/Desktop/test.txt";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("lock_user Plan was run");
    }
	
	public void allowed_hosts() {
		final String dir = "/home/";
		final String file = "/bin/sed";
	    final String args = "-i \"s/ALL/#ALL/\" /etc/hosts.allow";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("allowed_hosts Plan was run");
    }
	
	public void not_allowed_hosts() {
		final String dir = "/home/";
		final String file = "/bin/sed";
	    final String args = "-i \"s/ALL/#ALL/\" /etc/hosts.deny";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("not_allowed_hosts Plan was run");
    }
	
	public void pass_max_days() {
		final String dir = "/home/";
		final String file = "/bin/sed";
	    final String args = "-i \"s/PASS_MAX_DAYS[\t| ]*[0-9]+/PASS_MAX_DAYS 541/\" /etc/login.defs";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("pass_max_days Plan was run");
    }
	
	public void min_days() {
		final String dir = "/home/";
		final String file = "/bin/sed";
	    final String args = "-i \"s/PASS_MIN_DAYS[\t| ]*[0-9]+/PASS_MIN_DAYS 541/\" /etc/login.defs";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("min_days Plan was run");
    }
	
	public void warn_age() {
		final String dir = "/home/";
		final String file = "/bin/sed";
	    final String args = "-i \"s/PASS_WARN_AGE[\t| ]*[0-9]+/PASS_WARN_AGE 541/\" /etc/login.defs";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
				//System.out(error.toString());
				}		
			});
		System.out.println("warn_age Plan was run");
    }
	
	private String[] toArgsArray(String file, String cmd) {
            // Create arguments list from a command line.
            int i = 0;
            int l = cmd.length();
            List<String> arr = new ArrayList<String>();
            arr.add(file);
            for (;;) {
                while (i < l && cmd.charAt(i) == ' ') i++;
                if (i >= l) break;
                String s = null;
                if (cmd.charAt(i) == '"') {
                    i++;
                    StringBuffer bf = new StringBuffer();
                    while (i < l) {
                        char ch = cmd.charAt(i++);
                        if (ch == '"') break;
                        if (ch == '\\' && i < l) ch = cmd.charAt(i++);
                        bf.append(ch);
                    }
                    s = bf.toString();
                }
                else {
                    int i0 = i;
                    while (i < l && cmd.charAt(i) != ' ') i++;
                    s = cmd.substring(i0, i);
                }
                arr.add(s);
            }
            return arr.toArray(new String[arr.size()]);
    }

    private void exit(Throwable x) {
        if (!test_suite.isActive(this)) return;
        test_suite.done(this, x);
    }
}
