package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response;

import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IToken;
import org.eclipse.tm.tcf.services.IFileSystem;
import org.eclipse.tm.tcf.services.IFileSystem.FileSystemException;
import org.eclipse.tm.tcf.services.IFileSystem.IFileHandle;

/**
 * This Reactive Measure will cause the kernel does not accept ICMP redirect packets.
 * ICMP redirects can be used to alter your routing tables, possibly to a malicious end.
 * proc/sys/net/ipv4/conf/all/accept_redirects's value must be 0
 */

public class RMTranDisableRedirectAcceptance implements IReactiveMeasures {
	static public String Name = "RM_TRANSIENT_DISABLE_REDIRECT_ACCEPTANCE";
	static public int Id = 20;
	static public double Risk_threshold = 0.6;
    
    private final IFileSystem files;
    private IFileHandle filehandle;



    public RMTranDisableRedirectAcceptance(IChannel channel) {
            
    	files = channel.getRemoteService(IFileSystem.class);

    }
    
	@Override
	public void start() {
		final String path = "/proc/sys/net/ipv4/conf/all/accept_source_route";
		final String strContent = "0";
		
		files.open(path, IFileSystem.TCF_O_CREAT | IFileSystem.TCF_O_TRUNC | IFileSystem.TCF_O_WRITE, null, new IFileSystem.DoneOpen() {
            public void doneOpen(IToken token, FileSystemException error, IFileHandle handle) {
            	if (error != null) {
            		System.out.println(error);
                }
                else {
                    filehandle = handle;
                    files.write(handle, 0, strContent.getBytes(), 0,1 , new IFileSystem.DoneWrite() {

							@Override
							public void doneWrite(IToken token,
									FileSystemException error) {
								if (error != null) {
									System.out.println(error);
	                            }
	                            else {
	                            	files.close(filehandle, new IFileSystem.DoneClose() {
										@Override
										public void doneClose(IToken token, FileSystemException error) {
											System.out.println(Name + " with " + Risk_threshold + " threshold was run");
										}
									});
	                            }
							}
                        	
                        });
                  }
            }
        });
		
	}
	
}
