package org.eclipse.linuxtools.lttng.SYSPRO.RiskAssessment;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;

import net.sourceforge.jFuzzyLogic.FIS;

import org.eclipse.core.runtime.Platform;
import org.eclipse.linuxtools.lttng.SYSPRO.ui.Application;
import org.eclipse.linuxtools.lttng.SYSPRO.ui.Connect;
import org.eclipse.ui.internal.util.BundleUtility;
import org.osgi.framework.Bundle;



public class RiskAssessment {
	
	int resourceNO = 0;
	int alertClassificationNO = 0;
	Connect c = new Connect();
	ResultSet rs = null;
	FIS fis_threat;
	FIS fis_RiskIndex;
	double[] resourceArray;
	double[] resourceMappingArray;
	double[] alertNumArray;
	double[] alertRiskImpactArray;
	double[] AcceptableArray;

	
	public RiskAssessment () {
		
		// Load from 'FCL' file
		Bundle bundle = Platform.getBundle(Application.PLUGIN_ID);
		URL fullPathString = BundleUtility.find(bundle, "fcl/RiskIndex.fcl");

		InputStream istream = null;
		try {
			istream = fullPathString.openStream();
		} catch (IOException e) {
			e.printStackTrace();
		}
		fis_RiskIndex = FIS.load(istream,true);
		// Error while loading?
		if( fis_RiskIndex == null ) { 
			System.err.println("Can't load file: '" 
					+ fullPathString + "'");
			// return;
		}

		fullPathString = BundleUtility.find(bundle, "fcl/Threat.fcl");

		istream = null;
		try {
			istream = fullPathString.openStream();
		} catch (IOException e) {
			e.printStackTrace();
		}
		fis_threat = FIS.load(istream,true);
		// Error while loading?
		if( fis_RiskIndex == null ) { 
			System.err.println("Can't load file: '" 
					+ fullPathString + "'");
			// return;
		}

		// Set Resource Array
		rs = null;
		try {
			rs = c.Query("SELECT COUNT(*) FROM Resource");
			rs.next();
			resourceNO = rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		resourceArray = new double[resourceNO+1];
		resourceMappingArray = new double[resourceNO+1];

		for (int i=1; i <= resourceNO; i++) {
			resourceArray[i] = 0;
		}

		int count = 1;
		try {
			rs = c.Query("select * from Resource");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		try {
			while (rs.next ())
			{
				String idVal = rs.getString ("Resource.Id");
				resourceMappingArray[count++] = Integer.parseInt(idVal);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		// Set Alert Classification Array
		rs = null;
		try {
			rs = c.Query("SELECT COUNT(*) FROM AlertClassification");
			rs.next();
			alertClassificationNO = rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		alertNumArray = new double[alertClassificationNO+1];
		AcceptableArray = new double[alertClassificationNO+1];

		for (int i=1; i <= alertClassificationNO; i++) {
			alertNumArray[i] = 0;
			alertRiskImpactArray[i] = 0;
		}

		// Set AcceptableArray
		int Rowcount = 1;

		try {
			rs = c.Query("Select * from AlertClassification ORDER BY Id ASC");
		} catch (SQLException e) {
			e.printStackTrace();
		}
		try {
			while (rs.next ())
			{
				int AcceptableVal = Integer.parseInt(rs.getString ("Acceptable"));

				AcceptableArray[Rowcount++]= AcceptableVal;

			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public double run(int resourceIdVal,int severityVal, double resourceValueVal, double vulnerabilityValueVal, int alertClassificationIdVal, double confidenceVal) {
		double riskIdex, threatEffect;
		double freq_per_alert;
		double TotalRiskImpact= 0.0;

		freq_per_alert = CalculateFreq_Per_Alert(alertClassificationIdVal, confidenceVal);
		alertNumArray[alertClassificationIdVal] += freq_per_alert;

		int k;
		for (k=1; k <= resourceNO + 1; k++) {
			if (resourceMappingArray[k] == resourceIdVal) {
				resourceArray[k] += freq_per_alert;
				break;
			}
		}
		
		// Calculate Threat Effect
		fis_threat.setVariable("severity", severityVal);
		fis_threat.setVariable("freq_per_resource", resourceArray[k]);
		fis_threat.setVariable("freq_per_alert", alertNumArray[alertClassificationIdVal]);
		fis_threat.evaluate();
		threatEffect = fis_threat.getVariable("threat").getLatestDefuzzifiedValue();

		// Calculate Risk Index
		fis_RiskIndex.setVariable("resource", resourceValueVal);
		fis_RiskIndex.setVariable("vulnerability", vulnerabilityValueVal);
		fis_RiskIndex.setVariable("threat",threatEffect);
		fis_RiskIndex.evaluate();
		riskIdex = fis_RiskIndex.getVariable("risk_index").getLatestDefuzzifiedValue();
		
		DecimalFormat df = new  DecimalFormat ("0.##");

		System.out.println(" T: " + df.format(threatEffect) + "(S:" + df.format(severityVal) + ", FPR:" + df.format(resourceArray[k]) + ", FPA:" + df.format(alertNumArray[alertClassificationIdVal]) + ")");
		System.out.println("Alert " + alertClassificationIdVal + " T: " + df.format(threatEffect) + " V: " +  df.format(vulnerabilityValueVal) +" R: " + df.format(resourceValueVal) + " --> RI: " + df.format(riskIdex));
		
		alertRiskImpactArray[alertClassificationIdVal]= riskIdex;
		
		for (k=1; k <= alertClassificationNO; k++) {
			TotalRiskImpact+= alertRiskImpactArray[k];
		}
		              
		return riskIdex;
		
	}
	
	private double CalculateFreq_Per_Alert(int index, double confidence) {
		double freq = 0;

		freq = (1/AcceptableArray[index]) * (confidence/100);
		return freq;	
	}
}
