/*
 * @(#)FSMTest.java	1.12 06/08/06
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.eclipse.linuxtools.lttng.SYSPRO.file ;

import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Input ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Action ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.ActionBase;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Guard ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.GuardBase;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateEngine ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateImpl ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateEngineFactory ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.FSM ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.FSMImpl;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.State;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.InputImpl;
import org.eclipse.linuxtools.lttng.SYSPRO.file.TraceEventHandlerFile;
import org.eclipse.linuxtools.lttng.SYSPRO.file.SyntheticEvent;

public class FileFSM extends FSMImpl {
    private final Thread[] etherealizer ;   // The actual etherealize operation 
					    // for this entry.  It is 
					    // represented as a Thread because
					    // the POA.deactivate_object never 
					    // waits for the completion.
    private final int[] counter ;	    // single element holder for counter 
    //private SyntheticEvent syntheticevent;
    
    private Long numberOfRead;
    private Long readBytes ;
    private Long numberOfWrite;
    private Long writeBytes;
    private boolean sequential;
    private long startingTs;
    private String operationType = "File";
    private String filename;
					    // accessed in actions
   // private final CondVar wait ;	    // accessed in actions

    //final POAImpl poa ;
    public Long getReadNo(){
    	return this.numberOfRead;
    }
    public Long getWriteNo(){
    	return this.numberOfWrite;
    }
    public Long getReadBytesNo(){
    	return this.readBytes;
    }
    public Long getWriteBytesNo(){
    	return this.writeBytes;
    }
    
    public void resetStat(){
    	this.numberOfRead =0L;
    	this.readBytes=0L;
    	this.numberOfWrite= 0L;
    	this.writeBytes=0L;
        this.sequential = true;
        this.startingTs = 0L;
        this.filename = "";
    }
    
    public long getStartingTs(){
    	return startingTs;
    }
    
    public String getFilename(){
    	return this.filename;
    }
    
    public String getSyntheticEventInformation(){
    	String output = "";
    	if ((this.readBytes + this.writeBytes) > 0)
    		output = " ("+ (this.readBytes + this.writeBytes)+" bytes)";
	
    	return output;
    }
     
    
    public String getSyntheticeventType(){
    	return this.operationType;
    }
    public int getSyntheticeventName(){
    	if (this.numberOfRead >0){
    		if (this.numberOfWrite >0){
    			//return "Read.Write File Operation"; 
    			return 2006;/*These numbers will be read from an input XML file*/
    		}
    		else{
    			//return (this.sequential)?"Sequential File Read":"Nonsequential File Read"; 
    			return (this.sequential) ? 2002 : 2003 ; /*These numbers will be read from an input XML file*/
    		}		
    	}
    	else{
    		if (this.numberOfWrite >0){
    			//return (this.sequential)?"Sequential File Write":"Nonsequential File Write";
    			return (this.sequential) ? 2004 : 2005 ; /*These numbers will be read from an input XML file*/
    		}
    		else{
    			//return "Open.Close File Operation";
    			return 2001;
    		}
    	}
    }
    
    /*public void SetSyntheticEventClass(SyntheticEvent syntheticevent) 
	{
	    this.syntheticevent = syntheticevent;
	}*/
    
    public static final State STARTING = new StateImpl( "STARTING" );
    public static final State OPENED = new StateImpl( "Opened" );
    public static final State READ   = new StateImpl( "Read" );
    public static final State WRITE    = new StateImpl( "Write" ) ;
    public static final State SEEK   = new StateImpl( "Seek" );
    public static final State CLOSED   = new StateImpl( "Closed" ){
   	//private Object syntheticevent;
	public void preAction( FSM fsm ) {
	    FileFSM entry = (FileFSM)fsm ;
	    //System.out.println("----------------- First Synthetic Event -----------------");
	    //entry.afterclose();
	    /*Thread etherealizer = entry.etherealizer[0] ;
	    if (etherealizer != null) 
		etherealizer.start() ;*/
	}
	public void postAction( FSM fsm ) {
	    FileFSM entry = (FileFSM)fsm ;
	   
	    //this.syntheticevent.Add();
	    //...
	}
    };
    
    static final Input FS_OPEN    	= new InputImpl( "file open event" ) ;
    static final Input FS_READ	    = new InputImpl( "file read event" ) ;
    static final Input FS_SEEK	    = new InputImpl( "file (l)lssek event" ) ;
    static final Input FS_READV	    = new InputImpl( "file readv event" ) ;
    static final Input FS_WRITE	    = new InputImpl( "file write event" ) ;
    static final Input FS_WRITEV	= new InputImpl( "file writev event" ) ;
    static final Input FS_CLOSE	    = new InputImpl( "file close event" ) ;
    static final Input FS_LANDA	    = new InputImpl( "automatic state change" ) ;
   

    private static Action	seekAction = new ActionBase( "seekAction" ) {
    	public void doIt( FSM fsm, Input in ) {
    	    FileFSM entry = (FileFSM)fsm ;
    	    entry.sequential = false;
    	    if (in.getValue().contains("read"))
    	    	entry.numberOfRead ++;
    	    if (in.getValue().contains("write"))
    	    	entry.numberOfWrite ++;
    	    
    	    //entry.readBytes= 
    	}
        } ;
    private static Action readIncrementAction = new ActionBase( "readincrement" ) {
	public void doIt( FSM fsm, Input in ) {
	    FileFSM entry = (FileFSM)fsm ;
	    entry.numberOfRead ++;
	    //entry.readBytes= 
	}
    } ;
    private static Action writeIncrementAction = new ActionBase( "writeincrement" ) {
    	public void doIt( FSM fsm, Input in ) {
    	    FileFSM entry = (FileFSM)fsm ;
    	    entry.numberOfWrite ++;
    	}
        } ;

    private static Action decrementAction = new ActionBase( "decrement" ) {
	public void doIt( FSM fsm, Input in ) {
	    FileFSM entry = (FileFSM)fsm ;
	    if (entry.counter[0] > 0)
		entry.counter[0]-- ;
	    //else
		//throw entry.poa.lifecycleWrapper().FileFSMDecZero() ;
	}
    } ;

    private static Action throwIllegalStateExceptionAction = new ActionBase( 
	"throwIllegalStateException" ) {
	public void doIt( FSM fsm, Input in ) {
	    throw new IllegalStateException( 
		"No transitions allowed from the CLOSE state" ) ;
	} 
    } ;

    private static Guard waitGuard = new GuardBase( "wait" ) {
	public Guard.Result evaluate( FSM fsm, Input in ) {
	    FileFSM entry = (FileFSM)fsm ;
	   /* try {
		;//entry.wait.await() ;
	    } catch (InterruptedException exc) {
		// XXX Log this
		// NO-OP
	    }*/

	    return Guard.Result.DEFERED ;
	}
    } ;

    private static class CounterGuard extends GuardBase {
	private int value ;

	public CounterGuard( int value ) 
	{
	    super( "counter>" + value ) ;
	    this.value = value ;
	}
	
	

	public Guard.Result evaluate( FSM fsm, Input in ) 
	{
	    FileFSM entry = (FileFSM)fsm ;
	    return Guard.Result.convert( entry.counter[0] > value ) ;
	}
    } ;

    private static GuardBase greaterZeroGuard = new CounterGuard( 0 ) ;
    private static Guard zeroGuard = new Guard.Complement( greaterZeroGuard ) ;
    private static GuardBase greaterOneGuard = new CounterGuard( 1 ) ;
    private static Guard oneGuard = new Guard.Complement( greaterOneGuard ) ;

    private static StateEngine engine ;

    static {
	engine = StateEngineFactory.create() ;

	//	    State,   Input,     Guard,			Action,		    new State

	engine.add( STARTING, FS_OPEN,				null,    OPENED	) ;
	engine.add( STARTING, FS_CLOSE,				null,    CLOSED	) ;
	engine.add( STARTING, FS_SEEK,				null,    SEEK	) ;
	engine.add( STARTING, FS_READ,				readIncrementAction,		    READ	) ;
	engine.add( STARTING, FS_READV,				readIncrementAction,		    READ	) ;
	engine.add( STARTING, FS_WRITE,				writeIncrementAction,		    WRITE	) ;
	engine.add( STARTING, FS_WRITEV,				null,		    WRITE	) ;
	
	
	
	engine.add( OPENED, FS_CLOSE,				null,    CLOSED	) ;
	engine.add( OPENED, FS_READ,				readIncrementAction,		    READ	) ;
	engine.add( OPENED, FS_READV,				readIncrementAction,		    READ	) ;
	engine.add( OPENED, FS_WRITE,				writeIncrementAction,		    WRITE	) ;
	engine.add( OPENED, FS_WRITEV,				writeIncrementAction,		    WRITE	) ;
	engine.setDefault( OPENED) ; //?

	engine.add( READ,  FS_SEEK,			null/*here we can have a counter or some stats on how much data read or write*/,		    SEEK	) ;
	engine.add( READ,  FS_READ,			readIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( READ,  FS_READV,			readIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( READ,  FS_WRITE,			writeIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( READ,  FS_WRITEV,			writeIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( READ,   FS_CLOSE,				null,	    CLOSED	) ;
	engine.setDefault( READ, throwIllegalStateExceptionAction, STARTING ) ;
	
	engine.add( WRITE,  FS_SEEK,			null/*here we can have a counter or some stats on how much data read or write*/,		    SEEK	) ;
	engine.add( WRITE,  FS_READ,			readIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( WRITE,  FS_READV,			readIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( WRITE,  FS_WRITE,			writeIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( WRITE,  FS_WRITEV,			writeIncrementAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( WRITE,  FS_CLOSE,				null,	    CLOSED	) ;
	engine.setDefault( WRITE, throwIllegalStateExceptionAction, STARTING ) ;
	
	
	engine.add( SEEK,  FS_SEEK,			null/*here we can have a counter or some stats on how much data read or write*/,		    SEEK	) ;
	engine.add( SEEK,  FS_READ,			seekAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( SEEK,  FS_READV,			seekAction/*here we can have a counter or some stats on how much data read or write*/,		    READ	) ;
	engine.add( SEEK,  FS_WRITE,			seekAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( SEEK,  FS_WRITEV,			seekAction/*here we can have a counter or some stats on how much data read or write*/,		    WRITE	) ;
	engine.add( SEEK,  FS_CLOSE,				null,	    CLOSED	) ;
	engine.setDefault( SEEK) ; //?
	
	//engine.add(CLOSED,  FS_CLOSE,				null,	    CLOSED	) ;
	
	engine.add(CLOSED,  FS_LANDA,				null,	    STARTING	) ;
	//engine.add(CLOSED,  FS_OPEN,          null, OPENED) ;
	
	
	//engine.setDefault( CLOSED, throwIllegalStateExceptionAction, CLOSED ) ;

	engine.done() ;
    }

    public FileFSM( )
    {
	super( engine, STARTING, false/*debug*/) ;
	etherealizer = new Thread[1] ;
	etherealizer[0] = null ;
	counter = new int[1] ;
	counter[0] = 0 ;
	numberOfRead = 0L;
    readBytes= 0L;
    numberOfWrite= 0L;
    writeBytes=0L;
    this.sequential = true;
    startingTs = 0L;
    filename = "";
	resetStat();
	//wait = new CondVar( poa.poaMutex, 
	//    ((ORB)poa.getORB()).poaConcurrencyDebugFlag ) ;
    }

    // Methods that drive the FSM: the real interface to this class
    // Most just call the doIt method, but startEtherealize needs
    // the etherealizer.
    public void startFileFSM() 
    { 
	//this.etherealizer[0] = etherealizer ;
	doIt( FS_OPEN ) ; 
    }
    
    //public void open(long ts) {updateStartTs(ts);doIt( FS_OPEN ) ; }
    public void open(long ts, String filename) {updateStartTs(ts);doIt( FS_OPEN ) ; this.filename = filename; /*this.filename += filename + ";";*/}
    public void read(long ts,long bytes) {
    	if (bytes >0) 
    		this.readBytes+=bytes;
    	updateStartTs(ts);
    	doIt( FS_READ ) ; 
    }
    public void write(long ts,long bytes) {if (bytes >0) this.writeBytes+=bytes; updateStartTs(ts);doIt( FS_WRITE ) ; }
    public void seek(long ts,long offset,long whence) {if (!(whence ==1 && (offset ==1||offset==0)))/*FIXME: do something based on offset and whence*/updateStartTs(ts);doIt(FS_SEEK); }
    public void close() { doIt( FS_CLOSE ) ;}
    public void afterClose() { doIt( FS_LANDA ) ; }
    public void updateStartTs(long ts){
    	if (startingTs == 0)
    		startingTs = ts;
    }
    
}