package org.eclipse.linuxtools.lttng.SYSPRO.model.state;

import java.util.EnumMap;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.linuxtools.lttng.SYSPRO.model.Task;
import org.eclipse.linuxtools.lttng.SYSPRO.model.Task.TaskState;


public abstract class StateInfo {

	public enum Field {
		PREV_PID, NEXT_PID, PARENT_PID, CHILD_PID,
		PID, TGID, CHILD_TGID, FILENAME, IP, FD, WRITE_FD, RET, SOCK,
		STATE, CPU_ID, SRC_ADDR, SRC_PORT, DST_ADDR, DST_PORT, OFFSET, ORIGIN, PROTOCOL, FAMILY, TYPE
	}

	private Task task;
	private TaskState taskState;
	protected long start;
	protected long end;
	protected Map<Field, Object> fieldInfo;

	public StateInfo() {
		fieldInfo = new HashMap<StateInfo.Field, Object>();
	}

	public void setTaskState(TaskState taskState) {
		this.taskState = taskState;
	}

	public TaskState getTaskState() {
		return taskState;
	}

	public void setStartTime(long start) {
		this.start = start;
	}

	public long getStartTime() {
		return start;
	}

	public void setEndTime(long end) {
		this.end = end;
	}

	public long getEndTime() {
		return end;
	}

	@Override
	public String toString() {
		//return "[state=" + taskState.toString() + ",pid=" + getTask().getProcessId() + "]";
		String output = "";
		for (Field field : fieldInfo.keySet()) {
			output = output + field + "= " + fieldInfo.get(field) +"  ";
		}
		return output;
	}

	public void setTask(Task task) {
		this.task = task;
	}	

	public Task getTask() {
		return task;
	}
	
	public long getDuration() {
		return end - start;
	}

	public void setField(Field key, Object value) {
		if (fieldInfo == null)
			fieldInfo = new EnumMap<Field, Object>(Field.class);
		fieldInfo.put(key, value);
	}

	public boolean containField(Field key) {
		if (fieldInfo == null || !fieldInfo.containsKey(key))
			return false;
		return true;
	}
	
	public Object getField(Field key) {
		if (fieldInfo == null || !fieldInfo.containsKey(key))
			return null;
		return fieldInfo.get(key);
	}
}
