/*
 * @(#)FSMTest.java	1.12 06/08/06
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.eclipse.linuxtools.lttng.SYSPRO.net ;

import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Input ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Action ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.ActionBase;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.Guard ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.GuardBase;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateEngine ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateImpl ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.StateEngineFactory ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.FSM ;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.FSMImpl;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.State;
import org.eclipse.linuxtools.lttng.SYSPRO.fsm.InputImpl;

public class NetFSM extends FSMImpl {
    private final Thread[] etherealizer ;   // The actual etherealize operation 
					    // for this entry.  It is 
					    // represented as a Thread because
					    // the POA.deactivate_object never 
					    // waits for the completion.
    private final int[] counter ;	    // single element holder for counter 
    //private SyntheticEvent syntheticevent;
    private Long saddr;
    private Long daddr;
    private Long source;
    private Long dest;
    
    private Long rcvNo;
    private Long sendNo;
    private Long rcvBytes ;
    private Long sendBytes;
    private Long noOfAck;
    private Long noOfSyn;
    private boolean sequential;
    private Long type;
    private Long fd;
	private Long startingTs;
	private Long family;
	private Long protocol;
	private Long transport_protocol;
	
    private String operationType = "Network";
	private static final long IPPROTO_TCP = 6;
	private static final long IPPROTO_UDP = 17;

	
    //final POAImpl poa ;
    public long getStartingTs(){
    	return startingTs;
    }
    public Long getRcvNo(){
    	return this.rcvNo;
    }
    public Long getSendNo(){
    	return this.sendNo;
    }
    public Long getRcvBytes(){
    	return this.rcvBytes;
    }
    public Long getSendBytes(){
    	return this.sendBytes;
    }
    public Long getPacketsCount(){
    	return this.rcvNo + this.sendNo;
    }
    public void resetStat(){
    	this.rcvNo = 0L;
    	this.sendNo= 0L;
    	this.rcvBytes= 0L;
    	this.sendBytes= 0L;
    	this.noOfAck= 0L;
    	this.noOfSyn = 0L;
    	this.dest = 0L;
    	startingTs = 0L;
    	type = 0L;
    	transport_protocol = 0L;
    }
    public String getSyntheticeventType(){
    	return this.operationType;
    }
    
    public String getThroughput(){

    	String ioCount = ""; 
    	if ((this.rcvNo + this.sendNo) > 0)
    		ioCount = " ("+ String.valueOf(this.rcvNo + this.sendNo) + " bytes)";    		
    	return ioCount;
    }
    public String getSyntheticEventInformation(){
    	String output = "";
    	String ioCount = ""; 
    	if (this.saddr != null)
    		output = this.saddr + ":" + this.source + " --> " + this.daddr + ":"+ this.dest ;
    	else if (this.transport_protocol != null)
    		output = "transpart protocol: "+this.transport_protocol.toString();
    	else
    		output = "protocol: " + this.protocol + ", type: " + this.type;
    		
    	return output;
    }
    
    public long getSyntheticeventName(){ //FIXME: here we need more investigation to detect the correct type of the network
    	//if (transport_protocol == IPPROTO_TCP){
    	if (dest != null)
    		if (dest == 80l)
    			return 2010L;
    		if (dest == 21l)
    			return 2012L; 
    		//if port == 80 then http connection
    	//return 2008L;
    	//}
    	//else if (transport_protocol == IPPROTO_UDP){
    		if (dest == 53l)
    			return 2011L;
        	//return 2009L;
    	//}
    	//else 
    		return 2007L;
    }
    
    public static final State CREATED = new StateImpl( "CREATED" );
    public static final State CLOSED = new StateImpl( "CLOSED" );
    public static final State LISTEN = new StateImpl( "LISTEN" );
    public static final State CONNECTIONREQUESTED   = new StateImpl( "CONNECTIONREQUESTED" );
    public static final State CONNECTIONESTABLISHED    = new StateImpl( "CONNECTIONESTABLISHED" ) ;
    public static final State DATATRANSFER   = new StateImpl( "DATATRANSFER" );
    public static final State CLOSING   = new StateImpl( "CLOSING" ){

	public void preAction( FSM fsm ) {
	    NetFSM entry = (NetFSM)fsm;
	}
	public void postAction( FSM fsm ) {
	    NetFSM entry = (NetFSM)fsm;
	}
    };
    
    static final Input START   	= new InputImpl( "start dumy packet" );
    static final Input SYN   	= new InputImpl( "syn packet" );
    static final Input SYNACK	    = new InputImpl( "syn ack packet" ) ;
    static final Input ACK	    = new InputImpl( "ack packet" );
    static final Input FIN	    = new InputImpl( "fin packet" );
    static final Input FINACK	    = new InputImpl( "fin ack packet" );
    static final Input SYNSENT    	= new InputImpl( "send syn packet" );
    static final Input SYNACKSENT	    = new InputImpl( "send syn ack packet" );
    static final Input ACKSENT    = new InputImpl( "send ack packet" );
    static final Input FINSENT	    = new InputImpl( "send fin packet" );
    static final Input FINACKSENT	    = new InputImpl( "send fin ack packet" );
    static final Input ILISTEN	= new InputImpl( "listen for a connection" );
    static final Input CONNECT	    = new InputImpl( "connect" );
    static final Input CREATE	    = new InputImpl( "create" );
    static final Input DATASEND	    = new InputImpl( "send packet" );
    static final Input DATARCV	    = new InputImpl( "receive packet" );
    static final Input RST	    = new InputImpl( "rst packet" );
    static final Input CLOSE	    = new InputImpl( "rst packet" );
   

    
    private static Action sendAction = new ActionBase( "sendincrement" ) {
	public void doIt( FSM fsm, Input in ) {
	    NetFSM entry = (NetFSM)fsm ;
	    entry.sendNo ++;
	    //entry.readBytes= 
	}
    } ;
    private static Action rcvAction = new ActionBase( "receiveincrement" ) {
    	public void doIt( FSM fsm, Input in ) {
    	    NetFSM entry = (NetFSM)fsm ;
    	    entry.rcvNo ++;
    	}
        } ;

    private static Action throwIllegalStateExceptionAction = new ActionBase( 
	"throwIllegalStateException" ) {
	public void doIt( FSM fsm, Input in ) {
	    throw new IllegalStateException( 
		"No transitions allowed from the CLOSE state" ) ;
	} 
    } ;

    private static class CounterGuard extends GuardBase {
	private int value ;

	public CounterGuard( int value ) 
	{
	    super( "counter>" + value ) ;
	    this.value = value ;
	}
	public Guard.Result evaluate( FSM fsm, Input in ) 
	{
	    NetFSM entry = (NetFSM)fsm ;
	    return Guard.Result.convert( entry.counter[0] > value ) ;
	}
    } ;

    private static GuardBase greaterZeroGuard = new CounterGuard( 0 ) ;
    private static Guard zeroGuard = new Guard.Complement( greaterZeroGuard ) ;
    private static GuardBase greaterOneGuard = new CounterGuard( 1 ) ;
    private static Guard oneGuard = new Guard.Complement( greaterOneGuard ) ;

    private static StateEngine engine ;

    static {
	engine = StateEngineFactory.create() ;

	engine.add( CLOSED, START,										null,    CLOSED);
	engine.add( CLOSED, ILISTEN,									null,    LISTEN);
	engine.add( CLOSED, SYN,										null,    CONNECTIONREQUESTED	) ;
	engine.add( CLOSED, DATASEND,									null,    DATATRANSFER	) ;
	engine.add( CLOSED, DATARCV,									null,    DATATRANSFER	) ;
	engine.add( CLOSED, FIN,										null,	CLOSING	) ;
	engine.add( CLOSED, FINSENT,									null,	CLOSING	) ;
	engine.setDefault( CLOSED) ;
	
	engine.add( LISTEN, SYN,									null,    CONNECTIONREQUESTED	) ;
	engine.add( LISTEN, SYNSENT,								null,    CONNECTIONREQUESTED	) ;
	engine.add( LISTEN, RST,									null,	 CLOSED	) ;
	
	engine.add( CONNECTIONREQUESTED, SYN,						null,    CONNECTIONREQUESTED	) ;
	engine.add( CONNECTIONREQUESTED, SYNSENT,					null,    CONNECTIONREQUESTED	) ;
	engine.add( CONNECTIONREQUESTED, SYNACK,					null,	 CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONREQUESTED, SYNACKSENT,				null,	 CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONREQUESTED, RST,						null,	CLOSED	) ;
	
	engine.add( CONNECTIONESTABLISHED, SYN,						null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, SYNACK,					null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, SYNSENT,					null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, SYNACKSENT,				null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, ACK,						null,	DATATRANSFER	) ;
	engine.add( CONNECTIONESTABLISHED, ACKSENT,					null,	DATATRANSFER	) ;
	engine.add( CONNECTIONESTABLISHED, RST,						null,	CLOSED	) ;
	engine.add( CONNECTIONESTABLISHED, FIN,						null,	CLOSING	) ;
	engine.add( CONNECTIONESTABLISHED, FINSENT,					null,	CLOSING	) ;
	
	
	engine.add( DATATRANSFER, DATASEND,							null,	DATATRANSFER	) ;
	engine.add( DATATRANSFER, DATARCV,							null,	DATATRANSFER	) ;
	engine.add( DATATRANSFER, ACK,								null,	DATATRANSFER	) ;
	engine.add( DATATRANSFER, ACKSENT,							null,	DATATRANSFER	) ;	
	engine.add( DATATRANSFER, FIN,								null,	CLOSING	) ;
	engine.add( DATATRANSFER, FINSENT,							null,	CLOSING	) ;
	engine.add( DATATRANSFER, RST,								null,	CLOSED	) ;
	
	engine.add( CLOSING, RST,									null,		CLOSED	) ;
	engine.add( CLOSING, FINSENT,								null,		CLOSED	) ;
	engine.add( CLOSING, FINACKSENT,							null,		CLOSED	) ;
	engine.add( CLOSING, ACK,									null,		CLOSED	) ;
	engine.add( CLOSING, ACKSENT,								null,		CLOSED	) ;
	
	
	engine.add( CLOSED, CREATE,										null,    CREATED);
	engine.add( CREATED, CONNECT,								null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, CONNECT,								null,	CONNECTIONESTABLISHED	) ;
	engine.add( CONNECTIONESTABLISHED, DATASEND,						null,	DATATRANSFER	) ;
	engine.add( CONNECTIONESTABLISHED, DATARCV,					null,	DATATRANSFER	) ;
	
	engine.add( CREATED, CLOSE,									null,	CLOSED	) ;
	engine.add( LISTEN, CLOSE,									null,	CLOSED	) ;
	engine.add( CLOSED, CLOSE,									null,	CLOSED	) ;
	engine.add( CONNECTIONREQUESTED, CLOSE,						null,	CLOSED	) ;
	engine.add( CONNECTIONESTABLISHED, CLOSE,					null,	CLOSED	) ;
	engine.add( DATATRANSFER, CLOSE,								null,	CLOSED	) ;
	engine.add( CLOSING, CLOSE,									null,		CLOSED	) ;
	
	engine.done() ;
    }

    public NetFSM(long ts)
    {
	super( engine, CLOSED, false) ;
	etherealizer = new Thread[1] ;
	etherealizer[0] = null ;
	counter = new int[1] ;
	counter[0] = 0 ;
	rcvNo = 0L;
    sendNo= 0L;
    rcvBytes= 0L;
    sendBytes= 0L;
    startingTs = ts;
	resetStat();
    }

    // Methods that drive the FSM: the real interface to this class
    // Most just call the doIt method, but startEtherealize needs
    // the etherealizer.
    public void startNetFSM()
    {
    	doIt( START ) ; 
    }
    public void synrcvd(long bytes) { doIt( SYN ) ; }
    public void synsent(long bytes) { 	doIt( SYNSENT ) ; }
    public void synacksent(long bytes) { 	doIt( SYNACKSENT ) ; }
    
    public void acksent(long bytes) { 	doIt( ACKSENT ) ; }
    
    public void rstrcvd(long bytes) {doIt(RST);}
    public void rstsend(long bytes) {doIt(RST);}
    
    
    public void finrcvd(long bytes) {doIt(FIN);}
    public void finsent(long bytes) { 	doIt( FINSENT ) ; }
    public void finacksent(long bytes) { 	doIt( FINACKSENT ) ; }
    
    public void create(long ts, long fd) { doIt( CREATE ) ; this.fd = fd; this.startingTs = ts; }
    public void create(long ts, long fd, long protocol, long family, long type) { 
		    	doIt( CREATE ) ; 
		    	this.fd = fd; 
		    	this.startingTs = ts; 
		    	this.family = family; 
		    	this.protocol = protocol; 
		    	this.type = type; 
    		}
    public void connect() {doIt(CONNECT);}
    public void connect(long saddr, long source, long daddr, long dest) {doIt(CONNECT); this.saddr = saddr; this.daddr = daddr; this.source = source; this.dest = dest;}
    public void close() {doIt(CLOSE);}
    
    public void datasend(long bytes) {doIt(DATASEND);};
    public void datarcvd(long bytes) { doIt( DATARCV);}
    public void listen() { doIt( ILISTEN);}
    
    
    
    public void send(long transport_protocol,long tot_len,long syn,long ack,long fin,long rst) 
    { 
    	if (transport_protocol == 6){
    		if (rst==1)
    			rstsend(tot_len);
    		else{
	    		if  (syn==1){
	    			if (ack==1)
	    				synacksent(tot_len);
	    			else
	    				synsent(tot_len);
	    		}else if  (fin==1){
	    			if (ack==1)
	    				finacksent(tot_len);
	    			else
	    				finsent(tot_len);
	    		}
	    		else if (ack==1)
	    			acksent(tot_len);
    		}		
       }
    }
    
    public void send(long tot_len) 
    { 
    	this.sendNo += tot_len;
    	doIt(DATASEND);
    }
    
    
    public void send(long transport_protocol,long tot_len) 
    { 
    	this.transport_protocol = transport_protocol;
    	this.sendNo += tot_len;
    }
    
    public void rcv(long tot_len) 
    { 
    	this.rcvNo += tot_len;
    	doIt(DATARCV);
    }
    
    public void rcv(long transport_protocol,long tot_len) 
    { 
    	this.transport_protocol = transport_protocol;
    	this.rcvNo += tot_len;
    }
    
    public void rcv(long transport_protocol,long tot_len,long syn,long ack,long fin,long rst) 
    { 
    	if (transport_protocol == 6){
    		if (rst==1)
    			rstsend(tot_len);
    		else{
	    		if  (syn==1){
	    			if (ack==1)
	    				synacksent(tot_len);
	    			else
	    				synsent(tot_len);
	    		}else if  (fin==1){
	    			if (ack==1)
	    				finacksent(tot_len);
	    			else
	    				finsent(tot_len);
	    		}
	    		else if (ack==1)
	    			acksent(tot_len);
    		}
    				
       }
    }
    
}