package org.eclipse.linuxtools.lttng.SYSPRO.net;

import java.net.NetworkInterface;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeMap;

import org.eclipse.linuxtools.lttng.jni.JniEvent;
import org.eclipse.linuxtools.lttng.jni.JniTrace;
import org.eclipse.linuxtools.lttng.SYSPRO.cpu.KernelProcess;
import org.eclipse.linuxtools.lttng.SYSPRO.cpu.TraceEventHandlerProcess;
import org.eclipse.linuxtools.lttng.SYSPRO.file.FileOperationList;
import org.eclipse.linuxtools.lttng.SYSPRO.file.SyntheticEvent;
import org.eclipse.linuxtools.lttng.SYSPRO.io.TraceEventHandlerBase;
import org.eclipse.linuxtools.lttng.SYSPRO.io.TraceHook;
import org.eclipse.linuxtools.lttng.SYSPRO.io.TraceReader;


public class TraceEventHandlerNet extends TraceEventHandlerBase {
	private static final Long SYSCALL_CONNECT = 42L; //sys_connect
	private static final Long IPPROTO_TCP = 6L; //sys_connect
	private static final Long IPPROTO_UDP = 17L; //sys_connect
	
	public String intToIp(long i) {
	    return ((i >> 24 ) & 0xFF) + "." +
	           ((i >> 16 ) & 0xFF) + "." +
	           ((i >>  8 ) & 0xFF) + "." +
	           ( i        & 0xFF);
	}
	
	public Long ipToInt(String addr) {
        String[] addrArray = addr.split("\\.");

        long num = 0;
        for (int i=0;i<addrArray.length;i++) {
            int power = 3-i;
            num += ((Integer.parseInt(addrArray[i])%256 * Math.pow(256,power)));
        }
        return num;
    }
	
	public enum Family{ //or Domain
		AF_UNSPEC, AF_LOCAL, AF_INET, AF_AX25, AF_IPX, AF_APPLETALK, AF_NETROM, AF_BRIDGE, AF_ATMPVC, AF_X25
		, AF_INET6, AF_ROSE, AF_DECnet, AF_NETBEUI, AF_SECURITY, AF_KEY, AF_NETLINK, AF_ROUTE, AF_PACKET, AF_ASH, AF_ECONET, AF_ATMSVC
		, AF_SNA, AF_IRDA, AF_PPPOX, AF_WANPIPE, AF_BLUETOOTH, AF_IUCV, AF_RXRPC, AF_MA	
	}
	
	public enum Type{
		SOCK_DUMMY, SOCK_STREAM, SOCK_DGRAM, SOCK_RAW, SOCK_RDM, SOCK_SEQPACKET
	}
	
	public enum Protocol{
		IPPROTO_IP,	IPPROTO_ICMP, IPPROTO_IGMP, IPPROTO_DUMMY3, IPPROTO_IPIP, IPPROTO_DUMMY5, IPPROTO_TCP, IPPROTO_DUMMY7, IPPROTO_EGP, IPPROTO_DUMMY9, IPPROTO_DUMMY10, IPPROTO_DUMMY11, IPPROTO_PUP, IPPROTO_DUMMY13, IPPROTO_DUMMY14, IPPROTO_DUMMY15, IPPROTO_DUMMY16, IPPROTO_UDP
	}
	
	HashMap <String,NetworkPacket> OpenConnections;
	HashMap <Long, Set<NetworkPacket>> PacketStack; //(pid, netpacket)
	HashMap <Long, Set<NetworkPacket>> PidNetPackets; //(pid, netpacket)
	TreeMap<Long, HashMap<Long, NetworkOperationList>> PidFdNetworkoperation;// <pid,NetworkOperationList>
	HashMap<Long, NetworkOperationList> FdNetworkFileoperation; //<pid,NetworkOperationList>
	HashMap <Long, HashMap <Long, String>> OpenSockets; // (pid, (fd, state of the socket))
	HashMap <Long, Long> SyscallID;
	public TraceEventHandlerNet() {
		super();
		/*hooks.add(new TraceHook("net", "socket_create"));
		hooks.add(new TraceHook("net", "socket_bind"));
		hooks.add(new TraceHook("net", "socket_connect"));
		hooks.add(new TraceHook("net", "socket_listen"));
		hooks.add(new TraceHook("net", "socket_accept"));
		hooks.add(new TraceHook("net", "tcpv4_rcv"));
		hooks.add(new TraceHook("net", "udpv4_rcv"));	
		hooks.add(new TraceHook("net", "dev_receive"));
		hooks.add(new TraceHook("net", "dev_xmit"));
		hooks.add(new TraceHook("net", "dev_xmit_extended"));
		hooks.add(new TraceHook("net", "tcpv4_rcv_extended"));
		hooks.add(new TraceHook("net", "udpv4_rcv_extended"));
		hooks.add(new TraceHook("fs", "close"));
		hooks.add(new TraceHook("kernel", "syscall_entry"));*/
	}
	
	@Override
	public void handleInit(TraceReader reader, JniTrace trace) {
		FdNetworkFileoperation = new HashMap<Long, NetworkOperationList>();
		PidFdNetworkoperation = new TreeMap<Long,HashMap<Long,NetworkOperationList>>();
		OpenSockets=  new HashMap <Long, HashMap <Long, String>>();
		PidNetPackets  = new HashMap <Long, Set<NetworkPacket>>();
		PacketStack = new HashMap <Long, Set<NetworkPacket>>();
		OpenConnections = new HashMap <String, NetworkPacket>();
		SyscallID = new HashMap<Long,Long>();
	}

	/*
	public void  handle_net_socket_create(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		Long family = (Long) event.parseFieldByName("family");
		Long type = (Long) event.parseFieldByName("type");
		Long protocol = (Long) event.parseFieldByName("protocol");
		String sock = event.parseFieldByName("sock").toString();
		Long fd = (Long) event.parseFieldByName("ret");
		
		NetworkOperationList netOpList;
		String operation = "socket create";
		String operand="family: "+Family.class.getEnumConstants()[family.intValue()].toString()+", type: "+Type.class.getEnumConstants()[type.intValue()].toString()+", protocol: "+Protocol.class.getEnumConstants()[protocol.intValue()].toString()+", sock: "+sock.toString();
		
		if (fd>1){
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_socket_bind(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		Long fd = (Long) event.parseFieldByName("fd");
		String umyaddr = event.parseFieldByName("umyaddr").toString();
		Long addrlen = (Long) event.parseFieldByName("addrlen");
		Long ret = (Long) event.parseFieldByName("ret");
		
		NetworkOperationList netOpList;
		String operation = "socket bind";
		String operand="fd: "+fd.toString()+", umyaddr: "+umyaddr.toString()+", addrlen: "+addrlen.toString()+", ret: "+ret.toString();
		
		if (ret==0){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_socket_connect(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long src = 0L;
		Long dst = 0L;
		Long srcPort = 0L;
		Long dstPort = 0L;
		Long protocol = 0L;
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		Long fd = (Long) event.parseFieldByName("fd");
		String uservaddr = event.parseFieldByName("uservaddr").toString();
		
		Long addrlen = (Long) event.parseFieldByName("addrlen");
		Long ret = (Long) event.parseFieldByName("ret");
		
		if (currentProcessID==5150&&fd ==3&&ret ==0){
			long iii;
			iii=0L;
		}
		
		NetworkOperationList netOpList;
		String operation = "socket connect";
		String operand="fd: "+fd.toString()+", addrlen: "+addrlen.toString()+", ret: "+ret.toString();
		
		
		if (ret>=0){ //FIXME. this is only for test!. it must be: if (ret == 0)
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);	
			
			Set<NetworkPacket> setofpackets = PidNetPackets.get(currentProcessID);
			if (setofpackets == null){
				setofpackets =  new HashSet();
			}
			if (PacketStack.containsKey(currentProcessID) && !PacketStack.get(currentProcessID).isEmpty()){
				NetworkPacket lastPacket = null;
				
				for (NetworkPacket np : PacketStack.get(currentProcessID)){
					
					setofpackets.add(new NetworkPacket(np.getTitle(), currentProcessID, fd, np.getSrc(), np.getSrcPort(), np.getDst(), np.getDstPort(), np.getProtocol(),np.getTotalLen(),np.getValue(), "Connected", np.getTime()));
					lastPacket = np;
					src = np.getSrc();
					dst = np.getDst();
					srcPort = np.getSrcPort();
					dstPort = np.getDstPort();
					protocol = np.getProtocol();
				}
				//FIXME: we can remove this line if we only want to show the packets.
				//setofpackets.add(new NetworkPacket(operation, currentProcessID, fd, lastPacket.getSrc(), lastPacket.getSrcPort(), lastPacket.getDst(), lastPacket.getDstPort(), lastPacket.getProtocol(),lastPacket.getTotalLen(),operand, "Connected", eventTs));
				
				lastPacket.setFd(fd);
				lastPacket.setPid(currentProcessID);
			//	OpenConnections.put(lastPacket.getDst().toString()+  lastPacket.getDstPort().toString()+  lastPacket.getSrc().toString()+ lastPacket.getSrcPort().toString(), new NetworkPacket(currentProcessID, fd, src,srcPort,dst,dstPort,protocol,"Connected",operand));
				OpenConnections.put(lastPacket.getSrc().toString()+  lastPacket.getSrcPort().toString()+  lastPacket.getDst().toString()+ lastPacket.getDstPort().toString(), new NetworkPacket(currentProcessID, fd, src,srcPort,dst,dstPort,protocol,"Connected",operand));
				
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, lastPacket.getSrc().toString()+  lastPacket.getSrcPort().toString()+  lastPacket.getDst().toString()+ lastPacket.getDstPort().toString());
				OpenSockets.put(currentProcessID, fd_socketstate);
				PacketStack.remove(currentProcessID);
			}
			else {
				addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, fd, 0L, 0L, 0L, 0L, 0L,0L,operand, "SemiConnected", eventTs));
				
			}
			PidNetPackets.put(currentProcessID, setofpackets);
		}	
	}
	public void  handle_net_socket_listen(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		Long fd = (Long) event.parseFieldByName("fd");
		Long backlog = (Long) event.parseFieldByName("backlog");
		Long ret = (Long) event.parseFieldByName("ret");
		
		NetworkOperationList netOpList;
		String operation = "socket listen";
		String operand="fd: "+fd.toString()+", backlog: "+backlog.toString()+", ret: "+ret.toString();
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID, "netowrk", operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		netOpList.getFSM().listen();
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (ret==0){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_socket_accept(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		Long fd = (Long) event.parseFieldByName("fd");
		String upeer_sockaddr = event.parseFieldByName("upeer_sockaddr").toString();
		String upeer_addrlen = event.parseFieldByName("upeer_addrlen").toString();
		Long gs = (Long) event.parseFieldByName("flags");
		Long ret = (Long) event.parseFieldByName("ret");
		
		NetworkOperationList netOpList;
		String operation = "socket accept";
		String operand="fd: "+fd.toString()+", upeer_sockaddr: "+upeer_sockaddr.toString()+", upeer_addrlen: "+upeer_addrlen.toString()+", flags: "+gs.toString()+", ret: "+ret.toString();
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID, "netowrk", operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (ret >= 0){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_dev_xmit(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		String skb = event.parseFieldByName("skb").toString();
		Long protocol = (Long) event.parseFieldByName("protocol");
		Long fd = 1L;
		
		NetworkOperationList netOpList;
		String operation = "dev_xmit";
		String operand="skb: "+skb+", protocol: "+protocol.toString();
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (fd>=1){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_dev_receive(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		String skb = event.parseFieldByName("skb").toString();
		Long protocol = (Long) event.parseFieldByName("protocol");
		Long fd = 1L;
		
		NetworkOperationList netOpList;
		String operation = "dev_receive";
		String operand="skb: "+skb+", protocol: "+protocol.toString();
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (fd>=1){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_tcpv4_rcv(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		String skb = event.parseFieldByName("skb").toString();
		//Long protocol = (Long) event.parseFieldByName("protocol");
		Long fd = 1L;
		
		NetworkOperationList netOpList;
		String operation = "tcpv4_rcv";
		String operand="skb: "+skb;
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (fd>=1){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_net_udpv4_rcv(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;

		String skb = event.parseFieldByName("skb").toString();
		//Long protocol = (Long) event.parseFieldByName("protocol");
		Long fd = 1L;
		
		NetworkOperationList netOpList;
		String operation = "udpv4_rcv";
		String operand="skb: "+skb;
		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		if (fd>=1){
			//netOpList.getFSM().open();
			HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
			if (fd_socketstate == null){
				fd_socketstate= new HashMap <Long, String>();
				fd_socketstate.put(fd, operation);				
			}
			else
				fd_socketstate.put(fd, operation);
			OpenSockets.put(currentProcessID, fd_socketstate);
		}	
	}
	public void  handle_fs_close(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;
		Long fd = (Long) event.parseFieldByName("fd");		
		NetworkOperationList netOpList;
		String operation = "close";
		String operand="fd: "+fd.toString();
		if (OpenSockets.containsKey(currentProcessID)){
			if (!OpenSockets.get(currentProcessID).containsKey(fd))
				return;
		}
		else
			return;		
		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID, "netowrk", operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
	}
	public void  handle_kernel_syscall_entry(TraceReader reader, JniEvent event) {
		Long syscall_id = (Long) event.parseFieldByName("syscall_id");
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;
		
		SyscallID.put(currentProcessID, syscall_id);
			
	}

	public void handle_net_dev_xmit_extended(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;
		Long network_protocol = (Long) event.parseFieldByName("network_protocol");
		Long transport_protocol = (Long) event.parseFieldByName("transport_protocol");
		Long saddr = (Long) event.parseFieldByName("saddr");
		Long daddr = (Long) event.parseFieldByName("daddr");
		Long tot_len = (Long) event.parseFieldByName("tot_len");
		Long ihl = (Long) event.parseFieldByName("ihl");
		Long source = (Long) event.parseFieldByName("source");
		Long dest = (Long) event.parseFieldByName("dest");
		Long seq = (Long) event.parseFieldByName("seq");
		Long ack_seq = (Long) event.parseFieldByName("ack_seq");
		Long doff = (Long) event.parseFieldByName("doff");
		Long ack = (Long) event.parseFieldByName("ack");
		Long rst = (Long) event.parseFieldByName("rst");
		Long syn = (Long) event.parseFieldByName("syn");
		Long fin = (Long) event.parseFieldByName("fin");
		NetworkOperationList netOpList;
		String operation = "dev_xmit_extended";
		
		String operand=saddr + ":" + source+" --> "+daddr + ":" + dest +", syn=" + syn + " ack=" + ack + " fin=" + fin+" seq=" +seq +" ack_seq="+ack_seq+" rst="+rst+" len="+tot_len+" doff="+doff+" ihl="+ihl;
		Long fd= 1L;

		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		
		HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
		if (fd_socketstate == null){
			fd_socketstate= new HashMap <Long, String>();
			fd_socketstate.put(fd, operation);				
		}
		else
			fd_socketstate.put(fd, operation);
		OpenSockets.put(currentProcessID, fd_socketstate);
		
		Set<NetworkPacket> setofpackets = null;
		NetworkPacket newnp = null;
		Long currentFd = 0L;
		
		if (OpenConnections.containsKey(saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString())){
		
			newnp = OpenConnections.get(saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString());
			
			currentFd = newnp.getFd();
			currentProcessID = newnp.getPid();
		
			if (currentFd >1){
				setofpackets = PidNetPackets.get(currentProcessID);
				if (setofpackets == null)
					setofpackets =  new HashSet();
				
				setofpackets.add(new NetworkPacket(operation, newnp.getPid(),newnp.getFd(),saddr,source,daddr,dest,transport_protocol,tot_len, operand,"Connected", eventTs));

				
				PidFdNetworkoperation.get(newnp.getPid()).get(newnp.getFd()).getFSM().send(transport_protocol, tot_len, syn, ack, fin, rst);
			}
			else{
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT){
					addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,transport_protocol,tot_len, operand, "Unkown", eventTs));
					
				}
			}
			 newnp = null;
		}
		else{
			boolean found = false;
			if (PacketStack.containsKey(currentProcessID))
				for (NetworkPacket np : PacketStack.get(currentProcessID))
					if (np.getStatus() == "SemiConnected"){
						currentProcessID = np.getPid();
						fd = np.getFd();
						setofpackets = PidNetPackets.get(currentProcessID);
						if (setofpackets == null)
							setofpackets =  new HashSet();				
						setofpackets.add(new NetworkPacket(np.getTitle(), np.getPid(), np.getFd(), saddr,source,daddr,dest,transport_protocol,tot_len,operand,"Connected", np.getTime()));
						
						PidFdNetworkoperation.get(np.getPid()).get(np.getFd()).getFSM().send(transport_protocol, tot_len, syn, ack, fin, rst);
						OpenConnections.put(saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString(), new NetworkPacket(currentProcessID, fd, saddr,source,daddr,dest,transport_protocol,"Connected",operand));
						fd_socketstate= new HashMap <Long, String>();
						fd_socketstate.put(np.getFd(), saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString());
						OpenSockets.put(currentProcessID, fd_socketstate);
						removeFromPacketStack(currentProcessID, np);
						found = true;
						break;
					}
			if (!found){
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT){
					
					addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,transport_protocol, tot_len,operand,"unknown", eventTs));
					OpenConnections.put(saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString(), new NetworkPacket(currentProcessID, 0L, saddr,source,daddr,dest,transport_protocol,"unknown",operand));
					
				}
				else {
					setofpackets = PidNetPackets.get(currentProcessID);
					if (setofpackets == null)
						setofpackets =  new HashSet();
					setofpackets.add(new NetworkPacket(operation, currentProcessID, fd, saddr,source,daddr,dest,transport_protocol,tot_len,operand,"unknown", eventTs));
					
					PidFdNetworkoperation.get(currentProcessID).get(fd).getFSM().send(transport_protocol, tot_len, syn, ack, fin, rst);
					PidNetPackets.put(currentProcessID, setofpackets);
				}
			}
		}
	}
	public void addToPacketStack(Long pid, NetworkPacket pkt){
		Set<NetworkPacket>  netStackPackets= PacketStack.get(pid);
		if (netStackPackets == null)
			netStackPackets =  new HashSet();
		netStackPackets.add(pkt);
		PacketStack.put(pid, netStackPackets);
	}
	public void removeFromPacketStack(Long pid, NetworkPacket pkt){
		Set<NetworkPacket> netStackPackets= PacketStack.get(pid);
		if (netStackPackets == null)
			netStackPackets =  new HashSet();
		netStackPackets.remove(pkt);
		PacketStack.put(pid, netStackPackets);
		
	}
	
	public void handle_net_tcpv4_rcv_extended(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;
		Long saddr = (Long) event.parseFieldByName("saddr");
		Long daddr = (Long) event.parseFieldByName("daddr");
		Long tot_len = (Long) event.parseFieldByName("tot_len");
		Long ihl = (Long) event.parseFieldByName("ihl");
		Long source = (Long) event.parseFieldByName("source");
		Long dest = (Long) event.parseFieldByName("dest");
		Long seq = (Long) event.parseFieldByName("seq");
		Long ack_seq = (Long) event.parseFieldByName("ack_seq");
		Long doff = (Long) event.parseFieldByName("doff");
		Long ack = (Long) event.parseFieldByName("ack");
		Long rst = (Long) event.parseFieldByName("rst");
		Long syn = (Long) event.parseFieldByName("syn");
		Long fin = (Long) event.parseFieldByName("fin");
		
		NetworkOperationList netOpList;
		String operation = "tcpv4_rcv_extended";
		
		String operand=  saddr + ":" + source+" --> "+daddr + ":" + dest +", syn=" + syn + " ack=" + ack + " fin=" + fin+" seq=" +seq +" ack_seq="+ack_seq+" rst="+rst+" len="+tot_len+" doff="+doff+" ihl="+ihl;
		Long fd= 1L;

		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);

		HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
		if (fd_socketstate == null){
			fd_socketstate= new HashMap <Long, String>();
			fd_socketstate.put(fd, operation);				
		}
		else
			fd_socketstate.put(fd, operation);
		OpenSockets.put(currentProcessID, fd_socketstate);

		Set<NetworkPacket> setofpackets = null;
		NetworkPacket newnp = null;
		Long currentFd = 0L;
		setofpackets = PidNetPackets.get(currentProcessID);	
		if (setofpackets == null)
			setofpackets =  new HashSet();
		
		if (OpenConnections.containsKey(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString())){

			newnp = OpenConnections.get(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString());
			currentFd = newnp.getFd();
			currentProcessID = newnp.getPid();

			if (currentFd >1){
				setofpackets = PidNetPackets.get(currentProcessID);
				if (setofpackets == null)
					setofpackets =  new HashSet();
				setofpackets.add(new NetworkPacket(operation, newnp.getPid(),currentFd,saddr,source,daddr,dest, IPPROTO_TCP ,tot_len,operand,"Connected", eventTs));
				//if (currentProcessID > 0)
				PidFdNetworkoperation.get(newnp.getPid()).get(currentFd).getFSM().rcv(6L, tot_len, syn, ack, fin, rst);
				PidNetPackets.put(currentProcessID, setofpackets);
			}
			else{
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT){
					addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,IPPROTO_TCP,tot_len, operand,"Unkown", eventTs));
					
				}
			}
		}
		else{
			boolean found = false;
			if (PacketStack.containsKey(currentProcessID))
				for (NetworkPacket np : PacketStack.get(currentProcessID))
					if (np.getStatus() == "SemiConnected"){
						currentProcessID = np.getPid();
						fd = np.getFd();
						setofpackets = PidNetPackets.get(currentProcessID);
						if (setofpackets == null)
							setofpackets =  new HashSet();
						setofpackets.add(new NetworkPacket(np.getTitle(), np.getPid(), np.getFd(), saddr,source,daddr,dest,IPPROTO_TCP,tot_len,operand,"Connected", np.getTime()));
						//System.out.println("rcv-3  :"+currentProcessID+", fd:"+np.getFd()+",time="+eventTs+",newnp.pid="+np.getPid());						
						PidFdNetworkoperation.get(np.getPid()).get(np.getFd()).getFSM().rcv(6L, tot_len, syn, ack, fin, rst);
						
						OpenConnections.put(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString(), new NetworkPacket(np.getPid(), np.getFd(), saddr,source,daddr,dest,IPPROTO_TCP,"Connected",operand));
						
						fd_socketstate= new HashMap <Long, String>();
						fd_socketstate.put(np.getFd(), saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString());
						OpenSockets.put(currentProcessID, fd_socketstate);
						removeFromPacketStack(currentProcessID,np);
						found = true;
						break;
					}
			if (!found){
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT){
					addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,IPPROTO_TCP,tot_len, operand,"unknown", eventTs));
					OpenConnections.put(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString(), new NetworkPacket(currentProcessID, 0L, saddr,source,daddr,dest,IPPROTO_TCP,"unknown",operand));
				}
				else{
					setofpackets = PidNetPackets.get(currentProcessID);
					if (setofpackets == null)
						setofpackets =  new HashSet();
					setofpackets.add(new NetworkPacket(operation, currentProcessID, fd, saddr,source,daddr,dest,IPPROTO_TCP,tot_len, operand,"unknown", eventTs));
					
					PidFdNetworkoperation.get(currentProcessID).get(fd).getFSM().rcv(6L, tot_len, syn, ack, fin, rst);
					PidNetPackets.put(currentProcessID, setofpackets);
					//OpenConnections.put(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString(), new NetworkPacket(currentProcessID, 0L, saddr,source,daddr,dest,IPPROTO_TCP,"unknown",operand));
				}
				//System.out.println("here1:"+saddr.toString()+ ":" +source.toString()+ "-->" + daddr.toString()+ ":" +  dest.toString());
			}
		}
		//PidNetPackets.put(currentProcessID, setofpackets);
	}
	public void handle_net_udpv4_rcv_extended(TraceReader reader, JniEvent event) {
		String eventName = event.getMarkersMap().get(event.getEventMarkerId()).getName();
		Long currentProcessID;
		String currentProcessName="";
		Long eventTs = event.getEventTime().getTime();
		Long cpu = event.getParentTracefile().getCpuNumber();
		TraceEventHandlerProcess processHandler = (TraceEventHandlerProcess) reader.getHandler(TraceEventHandlerProcess.class);
		if (processHandler != null) {
			KernelProcess proc = processHandler.getCurrentProcess(cpu);
			if (proc == null){
				currentProcessID = 0L;
				currentProcessName ="";
			}
			else{
				currentProcessID = proc.getPid();
				currentProcessName = proc.getCmd();
			}
		}
		else
			currentProcessID = 0L;
		Long saddr = (Long) event.parseFieldByName("saddr");
		Long daddr = (Long) event.parseFieldByName("daddr");
		Long unicast = (Long) event.parseFieldByName("unicast");
		Long ulen = (Long) event.parseFieldByName("ulen");
		Long source = (Long) event.parseFieldByName("source");
		Long dest = (Long) event.parseFieldByName("dest");
		
		NetworkOperationList netOpList;
		String operation = "udpv4_rcv_extended";
		String operand="ulen="+ulen;
		Long fd= 1L;

		if (PidFdNetworkoperation.containsKey(currentProcessID))
		{
			FdNetworkFileoperation = PidFdNetworkoperation.get(currentProcessID);
			
			if (FdNetworkFileoperation.containsKey(fd)){
				netOpList = FdNetworkFileoperation.get(fd);
				netOpList.updateOperation(fd, operation, operand, eventTs);
			}
			else
				netOpList = new NetworkOperationList(fd,currentProcessID,"netowrk",operation, operand,currentProcessName,eventTs);
		}
		else
		{
			netOpList = new NetworkOperationList(fd, currentProcessID, "netowrk", operation, operand, currentProcessName, eventTs);
			FdNetworkFileoperation = new HashMap<Long,NetworkOperationList>();
		}
		FdNetworkFileoperation.put(fd, netOpList);
		PidFdNetworkoperation.put(currentProcessID, FdNetworkFileoperation);
		
		HashMap <Long, String> fd_socketstate = OpenSockets.get(currentProcessID);
		if (fd_socketstate == null){
			fd_socketstate= new HashMap <Long, String>();
			fd_socketstate.put(fd, operation);				
		}
		else
			fd_socketstate.put(fd, operation);
		OpenSockets.put(currentProcessID, fd_socketstate);
		Set<NetworkPacket> setofpackets = null;
		NetworkPacket newnp = null;
		Long currentFd = 0L;
	
		setofpackets = PidNetPackets.get(currentProcessID);	
		if (setofpackets == null)
			setofpackets =  new HashSet();
		
		if (OpenConnections.containsKey(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString())){
			newnp = OpenConnections.get(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString());
			currentFd = newnp.getFd();
			currentProcessID = newnp.getPid();
			if (currentFd >1){
				setofpackets = PidNetPackets.get(currentProcessID);
				if (setofpackets == null)
					setofpackets =  new HashSet();
				setofpackets.add(new NetworkPacket(operation, newnp.getPid(),newnp.getFd(),saddr,source,daddr,dest, IPPROTO_UDP,ulen,operand,"Connected", eventTs));
			}
			else{
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT)
					addToPacketStack(currentProcessID, new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,IPPROTO_UDP,ulen,operand, "Unkown", eventTs));
			}
		}
		else{
			boolean found = false;
			if (PacketStack.containsKey(currentProcessID))
				for (NetworkPacket np : PacketStack.get(currentProcessID))
					if (np.getStatus() == "SemiConnected"){
						setofpackets.add(new NetworkPacket(np.getTitle(), np.getPid(), np.getFd(), saddr,source,daddr,dest,IPPROTO_UDP,ulen,operand,"Connected", np.getTime()));
						setofpackets.add(new NetworkPacket(operation, np.getPid(), np.getFd(), saddr,source,daddr,dest,IPPROTO_UDP,ulen,operand,"Connected", eventTs));
						OpenConnections.put(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString(), new NetworkPacket(np.getPid(), np.getFd(), saddr,source,daddr,dest,IPPROTO_UDP,"Connected",operand));
						//System.out.println("here0:"+saddr.toString()+ ":" +source.toString()+ "-->" + daddr.toString()+ ":" +  dest.toString());
						fd_socketstate= new HashMap <Long, String>();
						fd_socketstate.put(np.getFd(), saddr.toString()+ source.toString()+ daddr.toString()+ dest.toString());
						OpenSockets.put(currentProcessID, fd_socketstate);
						removeFromPacketStack(currentProcessID, np);
						found = true;
						break;
					}
			if (!found){
				if (SyscallID.get(currentProcessID) == SYSCALL_CONNECT)
					addToPacketStack(currentProcessID,new NetworkPacket(operation, currentProcessID, 0L,saddr,source,daddr,dest,IPPROTO_UDP,ulen, operand,"unknown", eventTs));
				else
					setofpackets.add(new NetworkPacket(operation, currentProcessID, 1L, saddr,source,daddr,dest,IPPROTO_UDP,ulen,operand,"unknown", eventTs));
				OpenConnections.put(daddr.toString()+ dest.toString()+ saddr.toString()+ source.toString(), new NetworkPacket(currentProcessID, 0L, saddr,source,daddr,dest,IPPROTO_UDP,"unknown",operand));
			}
		}
		PidNetPackets.put(currentProcessID, setofpackets);
			
	}
*/
	@Override
	public void handleComplete(TraceReader reader) {
		
	}
	public TreeMap<Long,HashMap<Long,NetworkOperationList>> getNetworkOperationList(){
		return PidFdNetworkoperation;
	}
	public HashMap <Long, Set<NetworkPacket>> getNetworkPacketList(){
		return PidNetPackets;
	}

}
